@@ -78,7 +78,7 @@ def is_CJK(c):
     return False
 
 
-def prepare_seed(seed):
+def normalize_text(seed):
     # normalize
     seed = unicodedata.normalize('NFKD', unicode(seed))
     # lower
@@ -107,8 +107,7 @@ class Mnemonic(object):
     # Mnemonic phrase uses a hash based checksum, instead of a wordlist-dependent checksum
 
     def __init__(self, lang=None):
-        if lang in [None, '']:
-            lang = i18n.language.info().get('language', 'en')
+        lang = lang or 'en'
         print_error('language', lang)
         filename = filenames.get(lang[0:2], 'english.txt')
         path = os.path.join(os.path.dirname(__file__), 'wordlist', filename)
@@ -127,7 +126,8 @@ class Mnemonic(object):
     @classmethod
     def mnemonic_to_seed(self, mnemonic, passphrase):
         PBKDF2_ROUNDS = 2048
-        mnemonic = prepare_seed(mnemonic)
+        mnemonic = normalize_text(mnemonic)
+        passphrase = normalize_text(passphrase)
         return pbkdf2.PBKDF2(mnemonic, 'electrum' + passphrase, iterations = PBKDF2_ROUNDS, macmodule = hmac, digestmodule = hashlib.sha512).read(64)
 
     def mnemonic_encode(self, i):
@@ -154,23 +154,16 @@ class Mnemonic(object):
             i = i*n + k
         return i
 
-    def check_seed(self, seed, custom_entropy):
-        assert is_new_seed(seed)
-        i = self.mnemonic_decode(seed)
-        return i % custom_entropy == 0
-
-    def make_seed(self, num_bits=128, prefix=version.SEED_PREFIX, custom_entropy=1):
-        n = int(math.ceil(math.log(custom_entropy,2)))
-        # bits of entropy used by the prefix
-        k = len(prefix)*4
-        # we add at least 16 bits
-        n_added = max(16, k + num_bits - n)
-        print_error("make_seed", prefix, "adding %d bits"%n_added)
-        my_entropy = ecdsa.util.randrange( pow(2, n_added) )
+    def make_seed(self, num_bits=128, prefix=version.SEED_PREFIX):
+        # increase num_bits in order to obtain a uniform distibution for the last word
+        bpw = math.log(len(self.wordlist), 2)
+        n = int(math.ceil(num_bits/bpw)) * bpw
+        print_error("make_seed", prefix, "adding %d bits"%n)
+        my_entropy = ecdsa.util.randrange(pow(2, n))
         nonce = 0
         while True:
             nonce += 1
-            i = custom_entropy * (my_entropy + nonce)
+            i = my_entropy + nonce
             seed = self.mnemonic_encode(i)
             assert i == self.mnemonic_decode(seed)
             if is_old_seed(seed):
